/* TODO add an offset back... but beware, as it hads 
    some padding in the scrolling */
.fattable-h-scrollbar {
  padding: 0;
  background-color: transparent;
  position: absolute;
  height: auto;
  bottom: 0px;
  left: 0px;
  right: 0px;
  overflow-x: scroll;
  overflow-y: hidden;
}
.fattable-h-scrollbar > div {
  padding: 0 !important;
}
.fattable-v-scrollbar {
  padding: 0;
  position: absolute;
  background-color: transparent;
  width: auto;
  top: 0px;
  bottom: 0px;
  right: 0px;
  overflow-x: hidden;
  overflow-y: scroll;
}
.fattable-v-scrollbar > div {
  padding: 0 !important;
}
.fattable {
  overflow: hidden;
}
.fattable ::-webkit-scrollbar {
  width: 10px;
  height: 10px;
}
.fattable ::-webkit-scrollbar-track {
  -webkit-border-radius: 5px;
  border-radius: 5px;
  background: rgba(0, 0, 0, 0.1);
}
.fattable ::-webkit-scrollbar-thumb {
  -webkit-border-radius: 5px;
  border-radius: 5px;
  background: rgba(0, 0, 0, 0.2);
}
.fattable ::-webkit-scrollbar-thumb:hover {
  background: rgba(0, 0, 0, 0.4);
}
.fattable ::-webkit-scrollbar-thumb:window-inactive {
  background: rgba(0, 0, 0, 0.05);
}
.fattable-moving {
  cursor: move;
}
.fattable-viewport {
  overflow: hidden;
  position: absolute;
  left: 0;
  top: 0;
  bottom: 0;
  right: 0;
}
.fattable-viewport > div {
  padding: 2px;
  position: absolute;
  font-size: 12px;
  font-family: sans-serif;
  overflow: hidden;
  text-overflow: ellipsis;
}
.fattable-body-container {
  position: absolute;
  overflow: hidden;
  /* overflow-x:scroll;
       overflow-y:scroll;*/

  bottom: 0px;
  width: 100%;
}
.fattable-header-container {
  position: absolute;
  overflow: hidden;
  width: 100%;
  height: 100px;
}
.fattable-header-container .fattable-viewport {
  height: 100%;
}
.fattable-header-container .fattable-viewport > div {
  height: 100%;
}
