/*
###############################################################################
# Copyright (c) 2014-2019, Lawrence Livermore National Security, LLC.
# 
# Produced at the Lawrence Livermore National Laboratory
# 
# LLNL-CODE-666778
# 
# All rights reserved.
# 
# This file is part of Conduit. 
# 
# For details, see: http://software.llnl.gov/conduit/.
# 
# Please also read conduit/LICENSE
# 
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions are met:
# 
# * Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the disclaimer below.
# 
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the disclaimer (as noted below) in the
#   documentation and/or other materials provided with the distribution.
# 
# * Neither the name of the LLNS/LLNL nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL LAWRENCE LIVERMORE NATIONAL SECURITY,
# LLC, THE U.S. DEPARTMENT OF ENERGY OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
# DAMAGES  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 
###############################################################################
*/
.treemapnode {
  box-sizing:border-box;
  -moz-box-sizing:border-box;
  -webkit-box-sizing:border-box;
  border: dashed 0px black;
  font: 10px sans-serif;
  line-height: 12px;
  overflow: hidden;
  position: absolute;
  text-indent: 2px;
}

.treenode circle {
  cursor: pointer;
  fill: #fff;
  stroke: steelblue;
  stroke-width: 1.5px;
}

.treenode text {
  font-size: 11px;
}

.boldText {
  font-weight: bold;
}

path.link {
  fill: none;
  stroke: #ccc;
  stroke-width: 1.5px;
}

* {
  box-sizing:border-box;
  -moz-box-sizing:border-box;
  -webkit-box-sizing:border-box;
}

body {
  margin: 20px 20px 20px 20px;
  min-width: 1024px;
  font: 12px sans-serif;
}

.visualizer {
  margin: 2em auto;
}

.pure-g {
  align-content:flex-start;
}

div [class*='pure-u'] {
  padding: 1em 0px;
  border-style: solid none none solid;
  border-width: 5px medium medium 5px;
  border-color: white;
}

.heading {
  font: 16px sans-serif;
  font-weight: bold;
  text-align: center;
  background-color: #cccccc;
}

.scary-button {
  font-size: 150%;
  background: rgb(202, 60, 60);
  color: white;
  text-shadow: 0 1px 1px rgba(0, 0, 0, 0.2);
}

#main {
  height:100vh;
  width:100vw;
}

#d3tree {
  padding: 0px 20px 0px 20px;
  position: relative;
  margin: 0 auto;
}

#d3treemap {
  padding: 0px 20px 0px 20px;
  position: relative;
  margin: 0 auto;
}

#searchResultsTableContainer {
  height: 175px;
}

#d3searchtable table {
  table-layout: fixed;
  border-collapse: collapse;
  width: 100%;
}

#d3searchtable th, #d3searchtable td {
  padding: 5px;
  text-align: left;
  overflow: hidden;
}

#d3searchtable thead tr {
  position: relative;
}

#d3searchtable tbody {
  display: block;
  overflow-y: auto;
  overflow-x: hidden;
  max-height: 160px;
}

#d3searchtable th:nth-child(1) {
  width: 50%;
}
#d3searchtable th:nth-child(2) {
  width: 16%;
}
#d3searchtable th:nth-child(3) {
  width: 17%;
}
#d3searchtable th:nth-child(4) {
  width: 17%;
}

.fattable-header-container {
    background-color: rgb(224, 224, 224);
    color: black;
    font-weight: bold;
}
.fattable-header-container > div > div {
    border-right: solid 1px rgb(203, 203, 203);
    padding: 7px;
}
.fattable-body-container {
    background-color: #ffffff;
}
.fattable-body-container > div > div {
    border-right: solid 1px rgb(203, 203, 203);
    padding: 7px;
}
.fattable-body-container > div > div.even {
    background-color: rgb(242, 242, 242);
}

.fattable-body-container > div > div.pending {
    background-color: rgba(255,0,0,0.06);
}

#valueTableContainer {
  height: 200px;
}

#d3valuetable {
  position: absolute;
  height:180px;
}

#goodbye {
  display: none;
  position: absolute;
  top: 0px;
  left: 0px;
  font-size: 300%;
  text-align: center;
  vertical-align: middle;
  height: 100%;
  width: 100%;
  line-height: 100vh;
}
